#!/www/python/bin/python
"""$URL: svn+ssh://svn/repos/trunk/dulcinea/bin/sitecheck.py $
$Id: sitecheck.py 26135 2005-02-15 21:36:23Z dbinger $
"""

import os
import sys
from dulcinea.traversal import check_exports

if len(sys.argv) != 2:
    print "Usage: sitecheck.py site"
    raise SystemExit

site = sys.argv[1]
print
print site
os.environ['SITE'] = site
check_exports()



"""
$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/ui/page_loader.py $
$Id: page_loader.py 26384 2005-03-17 22:45:06Z dbinger $
"""
from cStringIO import StringIO
from dulcinea import local
from dulcinea.site_util import get_root_directory
from dulcinea.ui.crumbs import get_exports
from dulcinea.ui.publisher import DulcineaPublisher
from popen2 import popen2
from pprint import pformat
from quixote import get_request
from quixote.directory import Directory
from quixote.http_request import HTTPRequest
from quixote.logger import DefaultLogger
from sancho.utest import UTest
import sys
import quixote.publish

class PrintLogger (DefaultLogger):

    def __init__(self):
        self.error_log = StringIO()
        self.access_log = None
        self.error_email = None

class TestPublisher (DulcineaPublisher):

    def __init__(self):
        DulcineaPublisher.__init__(self, logger=PrintLogger())
        # Make sure nothing gets committed.
        local.get_connection().commit = lambda: None
        local.get_connection().abort = lambda: None
        def expire_session():
            pass
        self.session_manager.expire_session = expire_session
        def del_session(self, key):
            pass
        self.session_manager.__class__.__delitem__ = del_session
        def set_actual_user(self, user):
            pass
        self.session_manager.session_class.set_actual_user = set_actual_user

    def process(self,
                PATH_INFO='/',
                input_string = '',
                SCRIPT_NAME='',
                SERVER_NAME='test@example.org',
                **kwargs):
        env = dict(PATH_INFO=PATH_INFO,
                   SCRIPT_NAME=SCRIPT_NAME,
                   SERVER_NAME=SERVER_NAME,
                   **kwargs)
        request = HTTPRequest(StringIO(input_string), env)
        self.process_request(request)
        return request


class PageTest(UTest):

    def __init__(self):
        db = local.open_database()
        self.publisher = TestPublisher()
        session_cookie_name = self.publisher.config.session_cookie_name
        session_key = "test"
        self.cookies='%s="%s"' % (session_cookie_name, session_key)
        request = HTTPRequest(None, dict(HTTP_COOKIE=self.cookies))
        self.publisher._set_request(request)
        session = local.get_session_manager()._create_session()
        self.user = self._get_admin_user()
        session.user = session.actual_user = session.last_user = self.user
        session.id = session_key
        local.get_session_manager()[session_key] = session
        assert session == local.get_session_manager().get_session()
        self.session = session
        UTest.__init__(self)
        quixote.publish._publisher = None
        db.close()

    def _get_admin_user(self):
        admin = local.get_user_db().get_admin()
        for user in admin.get_direct_grantees('staff'):
            if user.get_id().upper() != user.get_id():
                break
        return user

    def _get_paths(self, user):
        def _get_paths_inner(directory, base_path):
            get_request().environ['PATH_INFO'] = base_path + '/'
            exports = [export[0] for export in get_exports(directory)]
            exports.sort()
            for name in exports:
                objname = directory._q_translate(name)
                if objname is None:
                    obj = directory._q_lookup(name)
                else:
                    obj = getattr(directory, objname)
                if hasattr(obj, '_q_traverse'):
                    for path in _get_paths_inner(obj, base_path + '/' + name):
                        yield path
                else:
                    yield base_path +'/' + name
        request = HTTPRequest(None, dict(SCRIPT_NAME=''))
        self.publisher._set_request(request)
        request.session = local.get_session_manager().get_session()
        request.session.set_user(user)
        return [path for path in _get_paths_inner(get_root_directory(), '')]

    def _get_processed_request(self, path, follow_redirects=(301,),
                               **kwargs):
        """
        Returns a request after processing, after one redirect if necessary.
        """
        if 'HTTP_COOKIE' not in kwargs:
            kwargs['HTTP_COOKIE'] = self.cookies
        assert self.session.user is self.user
        request = self.publisher.process(path, **kwargs)
        assert self.publisher.session_manager.get(self.session.id) is self.session
        assert request.session is self.session
        assert request.session.user is self.user
        if request.response.get_status_code() in follow_redirects:
            location = request.response.get_header('location')
            path = '/' + location.split('/', 3)[-1] # strip http://server
            request = self.publisher.process(path, **kwargs)
        return request

    def _is_reportable(self, line):
        if 'Warning: ' in line:
            warning = line.split('Warning: ', 1)[1].strip()
            if warning in (
                '<table> lacks "summary" attribute',
                '<a> attribute "href" lacks value',
                '<form> attribute "action" lacks value',
                '<input> attribute "name" lacks value',
                'trimming empty <option>',
                'trimming empty <span>',
                '<textarea> proprietary attribute "wrap"',
                '<a> cannot copy name attribute to id', # starts with digit
                ):
                return False
            if warning.startswith("trimming empty <"):
                return False
            if (warning.endswith('" already defined') and
                warning.startswith('<a> anchor "')):
                return False
        return True

    def _expect(self, path, status=200, **kwargs):
        self.publisher.logger.error_log = StringIO()
        request = self._get_processed_request(path, **kwargs)
        assert request.session.user.is_admin(), request.session.user
        try:
            assert request.response.get_status_code() == status, (
                "%r (expected %r) %s" % (request.response.get_status_code(),
                                         status,
                                         path))
        except:
            print request.response.get_status_code(), path
            request.response.write(sys.stdout)
            if request.response.get_status_code() != status:
                print self.publisher.logger.error_log.getvalue()
            raise

    def _validate_html(self, request):
        if request.response.get_content_type() == 'text/html':
            s = StringIO()
            for chunk in request.response.generate_body_chunks():
                s.write(chunk)
            ins, outs = popen2("/usr/bin/env tidy -q -e 2>&1")
            outs.write(s.getvalue())
            outs.close()
            body_lines = s.getvalue().splitlines()
            errors = []
            for line in ins.readlines():
                if self._is_reportable(line):
                    splitline = line.split()
                    lineno = int(splitline[1])
                    colno = int(splitline[3])
                    if body_lines:
                        bodyline = body_lines[lineno-1]
                    else:
                        bodyline = ''
                    errors.append(
                        '%s    ->%s: %r\n\n' % (
                        line,
                        lineno,
                        bodyline[max(colno-20, 0):colno+100]))
            if errors:
                print (
                    "\nLoading %r:\n%s\n\n" % (
                    request.get_path(), ''.join(errors)))
                assert not errors, 'html errors found'
        return request

    def _crawl(self, user, omit_status=(200,)):
        """
        Collect information to use in a more general test.
        """
        expect = {}
        for path in self._get_paths(user):
            request = self._get_processed_request(path)
            if request.response.get_status_code() not in omit_status:
                expect[path] = request.response.get_status_code()
        assert 0, 'expect=\n%s' % pformat(expect)





#!/usr/bin/env python
"""
$URL: svn+ssh://svn/repos/trunk/dulcinea/bin/missed.py $
$Id: missed.py 26302 2005-03-08 14:06:55Z dbinger $

This script runs the python module named on the command line
with __name__ == '__main__' and reports lines not executed.
Works without change for ptl.

Put '#n' in lines that you don't want missed.py to report as
missed.
"""
from dulcinea.code_util import report_missed_lines, main

main(report_missed_lines)



"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/user.py $
$Id: user.py 26150 2005-02-16 15:00:46Z dbinger $
"""

import re, sha, binascii
from durus.persistent_dict import PersistentDict
from quixote.util import randbytes
from dulcinea.base import DulcineaPersistent
from dulcinea.sort import lexical_sort
from dulcinea.typeutils import typecheck


def hash_password(password):
    """Apply a one way hash function to a password and return the result."""
    return sha.new(password).hexdigest()


class DulcineaUser(DulcineaPersistent):
    """Information for a registered user.

    Instance attributes:
      id : string
        unique identifier for this user
      password_hash : string
        the hashed version of the user's password, created using
        the hash_password function
    """

    user_id_re = re.compile('^[-A-Za-z0-9_@.]*$')

    def __init__(self, user_id=None):
        self.id = None
        if user_id is not None:
            self.set_id(user_id)

        self.password_hash = None


    def __str__(self):
        return self.id or "*no id*"

    format = format_realname = __str__ # subclasses should override

    def set_id(self, user_id):
        typecheck(user_id, str)
        assert self.id is None, "'id' may only be set once"
        if not self.user_id_re.match(user_id):
            raise ValueError(
                'Invalid user ID %r: can only contain '
                'letters, numbers, and "-_@."' % user_id)
        self.id = user_id

    def get_id(self):
        return self.id

    def set_password(self, new_password, check=True):
        """Set the user's password to 'new_password'."""
        if check and self.check_new_password(new_password) != "":
            raise ValueError, 'invalid password'
        self.password_hash = hash_password(new_password)

    def valid_password(self, password):
        """Return true if the provided password is correct."""
        return self.password_hash == hash_password(password)

    def generate_password (self, length=6):
        """Set the password to a random value and return the new password."""
        password = binascii.b2a_base64(binascii.unhexlify(randbytes(length)))
        password = password[:length]
        self.set_password(password)
        return password

    def check_new_password(self, new_password):
        """(string) -> string
        Check if a new password is valid.  Returns the empty string if the
        password is okay otherwise returns a string that describes what is
        wrong with the entered password.
        """
        return ""

    def get_email(self):
        return None

    def format_realname(self):
        return ''

    def is_admin(self):
        """Override if you want superiority."""
        return False

    def is_null(self):
        return self.id == ''

    def is_disabled(self):
        return self.password_hash is None

    def __nonzero__(self):
        return not self.is_null()

    def is_system(self):
        return self.id == 'SYSTEM'


class DulcineaUserDatabase(DulcineaPersistent):
    """
    Class to hold all users in the system.  User IDs are always looked
    up in the user database, so you will generally not be able to use
    a user until it has been added to the user database.

    Instance attributes:
      users : PersistentDict { user_id:string : user:DulcineaUser }
        all known users
      motd : str
        message-of-the-day
    """

    user_class = DulcineaUser
    motd_is = str

    def __init__(self):
        self.users =  PersistentDict()
        self.motd = ''

    def get_all_users(self):
        return self.users.values()

    def get_users(self, sort=0):
        users = [user for user in self.users.itervalues()
                 if not (user.is_null() or
                         user.is_system() or
                         user.is_disabled())]
        if sort:
            users = lexical_sort(users)
        return users

    def get_disabled_users(self, sort=0):
        users = [user for user in self.users.itervalues()
                 if user.is_disabled() and not (user.is_null() or
                                                user.is_system())]
        if sort:
            users = lexical_sort(users)
        return users

    def get_matching_user(self, identifier):
        """(identifier : str) -> DulcineaUser | None

        Return a user with matching id or email address, or None
        if no such user is found.
        """
        user = self.get_user(identifier)
        if user:
            return user
        elif '@' in identifier:
            identifier = identifier.lower()
            for user in self.users.itervalues():
                if (user.get_email() or '').lower() == identifier:
                    return user
        return None

    def get_user(self, user_id):
        """Return the User object with id 'user_id', or None if no such user.
        """
        return self.users.get(user_id)

    def __getitem__(self, user_id):
        return self.users[user_id]

    def add_user(self, user):
        """Add User object 'user' to the user database.
        """
        assert not self.users.has_key(user.get_id())
        self.users[user.id] = user

    def get_admin(self):
        """Subclasses should override to return a PermissionManager
        """
        return None

    def get_null_user(self):
        user = self.users.get('')
        if user is None:
            user = self.user_class(user_id='')
            self.add_user(user)
        return user

    def get_motd(self):
        return self.motd

    def set_motd(self, motd):
        self.motd = motd or ''



#!/usr/bin/env python
"""
$URL: svn+ssh://svn/repos/trunk/dulcinea/bin/unused_imports.py $
$Id: unused_imports.py 26302 2005-03-08 14:06:55Z dbinger $

Reports imports of names that are not in code in the importing file.

Even though a name is not used in code in the importing file, the
import may still be necessary.  For example, names are sometimes
imported in module A for the purpose of being imported from A in other
modules, or for side effects.  Sancho scripts often use imported names
in test code that is stored in strings, and this script doesn't detect
that usage.  Quixote applications often import names that only appear
in _q_export lists, and this script will report them as unused
imports.  The bottom line is that you may use this script to locate
suspicious imports, but you still need to be careful about actually
deleting imports.
"""

from dulcinea.code_util import report_unused_imports, main

main(report_unused_imports)



"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/permission.py $
$Id: permission.py 25764 2004-12-14 19:27:32Z rmasse $

"""

from sets import Set
from durus.persistent_dict import PersistentDict
from dulcinea.spec import mapping
from dulcinea.typeutils import typecheck
from dulcinea.base import DulcineaPersistent

class PermissionSet(DulcineaPersistent):
    """
    Instance attributes:
      direct_grantees : { any }
        This permission is granted to any object in this set.
      indirect_grantees : { (pm:PermissionManager, prerequisite:str) }
        This permission is granted if 'pm' grants the permission
        'prerequisite' to the object.
    """

    def __init__(self):
        self.direct_grantees = Set()
        self.indirect_grantees = Set()

    def get_items(self):
        items = [(obj, None) for obj in self.direct_grantees]
        items += list(self.indirect_grantees)
        return items

    def get_direct_grantees(self):
        return list(self.direct_grantees)

    def directly_grants(self, obj):
        return obj in self.direct_grantees

    def grants(self, obj, _covered=None):
        if obj in self.direct_grantees:
            return True
        for permission_manager, prerequisite in self.indirect_grantees:
            if permission_manager.grants(prerequisite, obj, _covered):
                return True
        return False

    def add(self, obj, prerequisite=None):
        typecheck(prerequisite, str, allow_none=1)
        if prerequisite is not None:
            typecheck(obj, PermissionManager)
            self.indirect_grantees.add((obj, prerequisite))
        else:
            self.direct_grantees.add(obj)
        self._p_changed = 1

class PermissionManager:

    """
    Class attributes:
      valid_permissions : { permission:string : description:string }
        Subclasses should set this value to list and describe the permissions
        they may grant.

    Instance attributes:
      permissions : PersistentDict {permission:str : PermissionSet}
    """

    permissions_is = mapping({str:PermissionSet}, PersistentDict)

    valid_permissions = None

    def __init__(self):
        self.permissions = PersistentDict()

    def set_permission(self, permission, obj, prerequisite=None):
        typecheck(permission, str)
        permission_set = self.permissions.get(permission)
        if permission_set is None:
            self.permissions[permission] = permission_set = PermissionSet()
        permission_set.add(obj, prerequisite)

    def set_permission_items(self, permission, items):
        try:
            del self.permissions[permission]
        except KeyError:
            pass
        for obj, prerequisite in items:
            self.set_permission(permission, obj, prerequisite=prerequisite)

    def get_valid_permissions(self):
        return self.valid_permissions

    def get_permission_items(self, permission):
        typecheck(permission, str)
        permission_set = self.permissions.get(permission)
        if permission_set is None:
            return []
        else:
            return permission_set.get_items()

    def get_direct_grantees(self, permission):
        """(permission: str) -> [obj]

        Return the list of objects that are directly granted 'permission'.
        """
        permission_set = self.permissions.get(permission)
        if permission_set is None:
            return []
        else:
            return permission_set.get_direct_grantees()

    def get_permissions(self, obj):
        """(obj: any) -> [str]

        Return the list of permissions granted to 'obj'
        """
        return [ permission
                 for permission in self.permissions
                 if self.grants(permission, obj) ]

    def directly_grants(self, permission, obj):
        """(permission:str, obj) -> boolean

        Test if permission is granted directly to object.
        This is much faster than grants(), and often sufficient.
        """
        if permission not in self.permissions:
            return False
        else:
            return self.permissions[permission].directly_grants(obj)

    def grants(self, permission, obj, _covered=None):
        """(permission:str, obj, _covered:Set=None) -> bool

        Return true if 'permission' is granted (directly or indirectly)
        to 'obj'.
        """
        if permission not in self.permissions:
            return False
        else:
            _covered = _covered or Set()
            if self in _covered:
                return False
            else:
                _covered.add(self)
                return self.permissions[permission].grants(obj, _covered)



"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/stored_file.py $
$Id: stored_file.py 26468 2005-04-01 22:27:38Z rmasse $

Store a file in the filesystem.
"""

import os, string, grp
import errno
from datetime import datetime
from quixote.util import randbytes
from dulcinea.site_util import get_config_value
from dulcinea.spec import any
from dulcinea.typeutils import typecheck
from dulcinea.base import DulcineaPersistent
from dulcinea.user import DulcineaUser

ALLOWABLE_SPECIAL_CHARS = "-_.@"

def path_check(path):
    """(path:string) -> string | None

    Return a lower-case and normalized version of the 'path' after
    checking for allowable characters.
    Return None if path has '..' or any special characters
    other than what's in string.lowercase, string.digits, and
    the ALLOWABLE_SPECIAL_CHARS string
    """
    if '..' in path:
        return None
    path = path.lower()
    for letter in path:
        if (letter != '/' and letter not in string.lowercase and
            letter not in string.digits and
            letter not in ALLOWABLE_SPECIAL_CHARS):
            return None
    return path

MIME_TYPES = [("image/gif", "GIF image"),
              ("image/jpeg", "JPEG image"),
              ("image/png", "PNG image"),
              ("image/tiff", "TIFF image"),
              ("image/x-ms-bmp", "BMP image"),
              ("text/plain", "plain text"),
              ("text/html", "HTML text"),
              ("text/rtf", "richtext document"),
              ("application/pdf", "PDF document"),
              ("application/msword", "Word document"),
              ("application/vnd.ms-powerpoint", "PowerPoint document"),
              ("application/vnd.ms-excel", "Excel document"),
              ("application/postscript", "PostScript file"),
              ("application/octet-stream", "Binary data"),
              ("application/x-gtar", "gzipped tar file"),
              ("application/x-gzip", "gzipped file"),
              ("application/x-tar", "tar file"),
              ("application/zip", "zip file"),
              ]

def get_file_store():
    return get_config_value('file_store', '/var/tmp')


class StoredFile (DulcineaPersistent):
    """Class for reading from or writing to a stored file

    A StoredFile is a wrapper around a file in a place in the
    filesystem known as the 'file_store'.

    StoredFiles can be instantiated with a fresh source file to move a new
    file into the file_store, or instantiated by referencing an existing
    file in the file_store.

    Instance attributes:
      path : string
        Path to the physical file containing the stored file's data.
      filename : string
        Filename to use when downloading this file
      mime_type : string
        MIME type of the contents of this file.
      description : string
        Description of this file.
      owner : DulcineaUser
        The file's owner
      date : datetime
        timestamp
    """

    path_is = str
    filename_is = str
    mime_type_is = any(*[pair[0] for pair in MIME_TYPES])
    description_is = any(str, None)
    owner_is = DulcineaUser
    date_is = datetime

    def __init__(self, path):
        """(path: str)
        """
        typecheck(path, str)
        self.description = None
        self.owner = None
        self.date = datetime.now()

        # Check path for safety (no '..' components, that sort of thing)
        path = os.path.normpath(path)
        self.path = path_check(path)
        assert self.path, 'Bad name: %r' % path
        self.filename = os.path.basename(self.path)
        self.mime_type = None

    def get_path(self):
        return self.path

    def get_id(self):
        return self.path.replace('/', '')

    def open(self):
        """() -> file
        Returns a Python file object opened for reading.  This can be used to
        read the file's content.
        """
        return open(self.get_full_path())

    def chgrp(self, grp_id):
        """(grp_id : str | int)
        Change the category ownership of this file to the specified ID.
        """
        if isinstance(grp_id, str):
            grnam, grpass, gid, grmembers = grp.getgrnam(grp_id)
            grp_id = gid
        os.chown(self.get_full_path(), -1, grp_id)

    def set_mime_type(self, mime_type):
        """(mime_type:string)
        Sets the MIME type for the file.
        """
        # Check there's a single '/' in the MIME type
        if mime_type.count('/') != 1:
            raise ValueError, "Invalid MIME type %r" % mime_type
        self.mime_type = mime_type

    def get_mime_type(self):
        return self.mime_type

    def set_filename(self, filename):
        self.filename = filename

    def get_filename(self):
        return self.filename

    def set_description(self, description):
        self.description = description

    def get_description(self):
        return self.description

    def set_date(self, date):
        typecheck(date, datetime)
        self.date = date

    def get_date(self):
        return self.date

    def set_owner(self, owner):
        typecheck(owner, DulcineaUser)
        self.owner = owner

    def get_owner(self):
        return self.owner

    def get_size(self):
        """() -> int
        Return the size of the file, measured in bytes, or None if
        the file doesn't exist.
        """
        path = self.get_full_path()
        if not os.path.exists(path):
            return None
        stats = os.stat(path)
        return stats.st_size

    def get_full_path(self):
        """() -> string
        """
        return os.path.join(get_file_store(), self.path)

    def has_manage_access(self, user):
        return (user is self.owner or user.is_admin())


def get_random_path():
    h = randbytes(6)
    return '%s/%s/%s' % (h[-1], h[-3:-1], h[:-3])

def new_file(fp):
    root_directory = get_file_store()
    flags = os.O_WRONLY|os.O_CREAT|os.O_EXCL
    try:
        flags |= os.O_BINARY    # for Windows
    except AttributeError:
        pass
    for attempt in xrange(1000):
        path = get_random_path()
        full_path = os.path.join(root_directory, path)
        destdir = os.path.dirname(full_path)
        if not os.path.exists(destdir):
            os.makedirs(destdir, mode=0775)
        try:
            fd = os.open(full_path, flags)
        except OSError, exc:
            if exc.errno == errno.EEXIST:
                pass # destination exists, retry
            else:
                raise # some other error
        else:
            break
    else:
        raise RuntimeError("Unable to make temp file.")
    dest_fp = os.fdopen(fd, "wb")
    while 1:
        chunk = fp.read(10000)
        if not chunk:
            break
        dest_fp.write(chunk)
    fp.close()
    dest_fp.close()
    return StoredFile(path)



"""
$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/ui/test/utest_table.py $
$Id: utest_table.py 26213 2005-02-23 21:31:30Z dbinger $
"""
from quixote import enable_ptl
enable_ptl()

from dulcinea.ui.table import Table
from sancho.utest import UTest
from quixote import publish
from quixote.http_request import HTTPRequest

class Test (UTest):

    def a(self):
        table = Table()
        table.tbody_id = 'test_id'
        table.render()
        table.column(a=1)
        table.column(b=2)
        table.render()
        class Publisher:
            def get_request(self):
                return HTTPRequest(None, {})
        publish._publisher = Publisher()
        for x in range(1000):
            table.row(a=x, b=x)
        table.render()

if __name__ == '__main__':
    Test()



#!/www/python/bin/python

"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/test/utest_sort.py $
$Id: utest_sort.py 25817 2004-12-21 16:27:07Z dbinger $

Test for dulcinea.sort.
"""

from sancho.utest import UTest
from dulcinea.sort import sort, str_sort, lexical_sort, \
     attr_sort, lex_attr_sort, \
     method_sort, function_sort, natural_sort


class Thing:
    counter = 0

    def __init__(self, name):
        self.name = name
        Thing.counter += 1
        self.id = "T-%03d" % Thing.counter

    def __str__(self):
        return self.name

    def __cmp__(self, other):
        return cmp(self.id, other.id)

    def get_id(self):
        return self.id


def gen(*args):
    for arg in args:
        yield arg


class SortTest (UTest):

    def check_sort(self):
        words = ["foo", "BAR", "aBc-123", "blah!", "123-abc"]
        sorted = ["123-abc", "BAR", "aBc-123", "blah!", "foo"]
        assert sort(gen(*words)) == sorted
        words2 = words[:]
        assert sort(words2) == sorted
        assert sort(words) == sorted
        assert words == sorted

    def check_instances(self):
        thing1 = Thing("bob")
        thing2 = Thing("Tim")
        thing3 = Thing("123-abc")
        things = [thing2, thing1, thing3]
        assert sort(gen(thing2, thing1, thing3)) == [thing1, thing2, thing3]
        assert str_sort(things) == [thing3, thing2, thing1]
        assert lexical_sort(gen(thing1, thing2, thing3)) == [
            thing3, thing1, thing2]
        assert attr_sort(things, 'id') == [thing1, thing2, thing3]
        assert attr_sort(things, 'name') == [thing3, thing2, thing1]
        assert lex_attr_sort(things, 'name') == [thing3, thing1, thing2]
        assert method_sort(gen(thing2, thing1, thing3), 'get_id') == [
            thing1, thing2, thing3]
        def fun_get_id(x):
            return x.get_id()
        assert function_sort(gen(thing2, thing1, thing3), fun_get_id) == [
            thing1, thing2, thing3]

    def check_natural(self):
        nums = [1, 5, 9, 10, 15, 105]
        assert str_sort(gen(*nums)) == [1, 10, 105, 15, 5, 9]
        nums = map(str, nums)
        assert sort(nums[:]) == ["1", "10", "105", "15", "5", "9"]
        assert natural_sort(nums) == nums

        # Check that the promises made in the docstring hold.
        l = ["a1b", "a1", "a20", "a", "a0", "a1a", "a10", "a2"]
        assert natural_sort(l) == [
            "a", "a0", "a1", "a1a", "a1b", "a2", "a10", "a20"]
        l = ["x2-y7", "x8-y8", "x2-g8", "x2-y08"]
        assert natural_sort(l) == [
            "x2-g8", "x2-y7", "x2-y08", "x8-y8"]
        l = ["1.02", "1.001", "1.3", "1.002", "1.1", "1.010"]
        assert natural_sort(l) == [
            "1.001", "1.002", "1.010", "1.02", "1.1", "1.3"]


if __name__ == "__main__":
    SortTest()



#!/www/python/bin/python

"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/test/utest_spec.py $
$Id: utest_spec.py 25402 2004-10-20 22:29:40Z dbinger $

Test for dulcinea.spec.
"""

from sancho.utest import UTest
from dulcinea.base import DulcineaPersistent
from dulcinea.spec import spec, nspec, any, get_spec_doc, get_specs, \
     get_spec_problems, match, require, format_spec, interval, \
     string, sequence, eq, anything, mapping, equal, subclass, instance, \
     all, length, no, boolean, get_spec, get_spec_report, specify


class SpecTest (UTest):

    def check_match (self):
        assert match(1, int)
        assert not match(2.5, int)
        assert match(2.5, any(int, float))
        assert not match('s', any(int, float))
        assert not match('hello', sequence(int, str))
        assert match('hello', sequence(str, str))
        assert not match('hello', sequence(str, tuple))
        assert match('hello', str)
        assert match('hello', any('yes', 'no', 'hello', int))
        assert match(3, any('yes', 'no', 'hello', int))
        mydict = {'a': 1, 'b': None}
        assert match(mydict, dict)
        assert match(mydict, { str : object })
        assert not match(mydict, { int : object })
        assert match(mydict, { str : object, int : object })
        assert match((2,3), (int, int))
        assert match((1,), tuple)
        assert match((1,), sequence(object, tuple))
        assert match((1,2,3,4), sequence(object, tuple))
        assert match((1,2,3,'ok'), sequence(object, tuple))
        assert not match((1,2,3,'ok'), sequence(int, tuple))
        assert match(mydict, mapping({ any(str, int) : object }))
        assert not match(mydict, mapping({int:object}, dict))
        assert not match(mydict, mapping({str: int}))
        assert match(True, boolean)
        assert not match(3, boolean)
        assert match([3,'ok'], [int, str])
        assert not match([3,'ok', 3], [int, str])
        assert match(3, no(None))
        assert match([3, 4], [all(int, no(None))])
        assert not match(3, any)
        assert match(3, object)
        a=[2,3]
        b=[2,3]
        assert match(a, eq(a))
        assert not match(b, eq(a))
        assert match(a, equal(b))
        assert match('a', string(1))
        assert match('a', length(1))
        assert match('a', interval('a','c'))
        assert not match('b', all(length(1), string(2)))
        assert not match('b', all(length(2)))
        assert match(10, interval(1))
        assert match(1, interval(1))
        assert not match(0, interval(1))
        assert not match(0, interval(1,4))
        assert match(4, interval(1,4))
        assert not match(5, interval(1,4))
        assert match(0, interval(anything,4))
        assert match(0, instance('int'))
        assert not match(0, no(int))

        class foo (dict, UTest):
            pass
        assert not match(0, subclass(dict))
        assert match(foo, subclass(dict))
        assert not match(0, mapping())
        assert not match(0, mapping({int:int},foo))
        sequence().get_args()
        assert not match(0, instance('foo'))
        for s in ('equal(1)',
                  'eq(1)',
                  'string(4)',
                  'sequence(2, anything)',
                  'interval(2, 4)',
                  'interval(2)',
                  'interval()',
                  'interval(anything, 3)',
                  'None',
                  '(int, str)',
                  '[int]',
                  '{int: str}',
                  "'constant'",
                  ):
            assert format_spec(eval(s)) == s
        try:
            require(1, str)
            assert 0
        except TypeError: pass
        try:
            require(1, str, 'oops')
            assert 0
        except TypeError: pass
        assert not match(1, [1])
        assert not match([1], [str])
        assert not match([1, 'hello'], [str, str])
        assert not match(1, {})
        class mytest:
            def __call__ (self, value):
                return True
        assert match(1, mytest())

    def check_specified (self):
        class A:
            flavor_is = spec(any('sweet', 'sour'), 'the flavor')
            texture_is = spec(int, 'roughness')

        class B(A):
            color_is = spec(str)
            texture_is = nspec(str)

        self.a = A()
        self.b = B()
        self.b.color = self.b.texture = 'ok'
        specify(self.b, flavor='sweet')
        assert get_spec_problems(self.b) == []
        self.b.nonsense = 34
        assert len(get_spec_problems(self.b)) == 1
        del self.b.nonsense
        assert len(get_spec_problems(self.b)) == 0
        self.b.color = 42
        assert len(get_spec_problems(self.b)) == 1
        self.b.color = 'red'
        assert len(get_spec_problems(self.b)) == 0
        assert len(get_specs(B).values()) == 3
        def is_texture(value):
            return value in (3,4)
        class C(DulcineaPersistent):
            color_is = nspec(str, 'the color')
            texture_is = is_texture
        assert get_specs(C)
        assert get_spec_doc(C)
        nspec(None)
        nspec(int)
        nspec(any(int, str))
        get_spec(C, 'texture_is')
        class D:
            color_is = nspec(str, 'the color')
            texture_is = is_texture
        get_spec(D, 'texture_is')
        get_spec_problems(D())
        class E:
            pass
        get_spec_report([D()])
        get_spec_report([E()])
        get_spec_report([])

    def check_specify (self):
        class A:
            flavor_is = spec(any('sweet', 'sour'), 'the flavor')
            texture_is = spec(int, 'roughness')

        class B(A):
            color_is = spec(str)
            texture_is = nspec(str)
        b = B()
        specify(b, texture='ok', flavor='sour')
        c = B()
        try:
            specify(c, texture=1)
            assert 0
        except TypeError:
            pass
        try:
            specify(c, boo=1)
            assert 0
        except AttributeError:
            pass

if __name__ == "__main__":
    SpecTest()



"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/item.py $
$Id: item.py 26063 2005-02-11 14:39:59Z dbinger $
"""
from dulcinea.base import DulcineaPersistent
from dulcinea.spec import boolean
from dulcinea.timestamped import Timestamped, reverse_timestamp_sorted
from dulcinea.typeutils import typecheck
from dulcinea.util import datetime_to_int
from durus.persistent_dict import PersistentDict
from quixote import get_user

class Item(DulcineaPersistent, Timestamped):
    """An Item object implements a mixin API for a unique stored item
    retrievable by creation time.

    Instance attributes:
      title: string
        The title of this item
      key : string
        Some sort of unique ID
      approved : boolean
        Whether this item has been approved.
    """

    title_is = str
    key_is = str
    approved_is = boolean

    def __init__(self):
        assert self.__class__ is not Item, "abstract class"
        Timestamped.__init__(self)
        self.key = None
        self.title = None
        self.approved = False

    def __cmp__(self, other):
        typecheck(other, Item)
        return cmp(self.get_timestamp(), other.get_timestamp())

    def absorb(self, other):
        assert self.__class__ is other.__class__
        for attr_name in self.__dict__:
            if attr_name != 'key':
                self.__dict__[attr_name] = other.__dict__[attr_name]
        self._p_changed = 1

    def set_key(self, key):
        typecheck(key, str)
        assert self.key is None, "Item already has a key"
        self.key = key

    def get_key(self):
        return self.key

    def set_title(self, title):
        typecheck(title, str)
        self.title = title

    def get_title(self):
        return self.title

    def set_approved(self, approved):
        typecheck(approved, bool)
        self.approved = approved

    def is_approved(self):
        return self.approved

    def get_user(self):
        return None

    def changes_allowed(self):
        user = get_user()
        if not user:
            return False
        if not (user is self.get_user() or user.is_admin()):
            return False
        return True


class ItemFolder(DulcineaPersistent):
    """Stores a collection of Items and defines an API for storage
    and creation-time-based retrieval.

    Instance attributes:
      items : PersistentDict { string : Item }
    """

    def __init__(self):
        self.items = PersistentDict()

    def clear(self):
        self.items.clear()

    def add_item(self, item):
        """(item : Item)
        """
        typecheck(item, Item)
        i = 0
        while 1:
            key =  '%s-%s' % (datetime_to_int(item.get_timestamp()), i)
            if not self.items.has_key(key):
                break
            i += 1
        item.set_key(key)
        self.items[key] = item

    def delete_item(self, item):
        """(item : Item)
        """
        typecheck(item, Item)
        if item.get_key() is None:
            raise KeyError, "Item %r not in any database" % item
        if not self.items.has_key( item.key ):
            raise KeyError, "Item %r not in this database" % item
        del self.items[item.get_key()]

    def get_item(self, key):
        """(key : str) -> Item
        """
        return self.items[key]

    def get_items(self):
        """() -> [Item]
        """
        return self.items.values()

    def get_recent_items(self, count=None, include_unapproved=False):
        """(count : int = None, include_unapproved : bool = False) -> [Item]

        Return the 'count' most recent items.  If 'include_unapproved' is True,
        both un-approved and approved items are returned.
        The list returned is in reverse-chronological order by timestamp.
        """
        typecheck(count, int, allow_none=1)
        result = []
        for item in self.items.values():
            if item.is_approved() or include_unapproved:
                result.append(item)
        result = reverse_timestamp_sorted(result)
        if count is not None:
            result = result[:count]
        return result



#!/www/python/bin/python
"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/ui/test/utest_browse.py $
$Id: utest_browse.py 26533 2005-04-08 19:51:08Z rmasse $
"""
from quixote import enable_ptl ; enable_ptl()
from dulcinea import local_ui
from dulcinea.ui.browse import ArchiveDirectory, ImageArchiveDirectory
from dulcinea.ui.browse import IndexArchiveDirectory, format_browse_css
from dulcinea.util import get_module_directory
from quixote.http_request import HTTPRequest
from quixote.logger import DefaultLogger
from quixote.publish import Publisher
from sancho.utest import UTest
import dulcinea.ui
import quixote.publish
import sys

local_ui.header = lambda a : None
local_ui.footer = lambda a : None

def get_test_archive(type):
    test_directory = get_module_directory(dulcinea.ui) + '/test/'
    return test_directory + 'browse_test.%s' % type

class PrintLogger(DefaultLogger):

    def __init__(self):
        self.error_log = sys.stdout
        self.access_log = None
        self.error_email = None

def decorate (obj, content, title=None):
    return content

class ArchiveDirectoryTest (UTest):

    def _pre(self):
        quixote.publish._publisher = None

    def _post(self):
        quixote.publish._publisher = None

    def test_misc(self):
        format_browse_css()

    def test_archive_directory_with_zip(self):

        archive_dir = ArchiveDirectory(get_test_archive('zip'),
                                       decorate=decorate)
        publisher = Publisher(archive_dir, logger=PrintLogger())
        publisher._set_request(HTTPRequest(None, dict(SCRIPT_NAME='',
                                                      PATH_INFO='/')))

        assert archive_dir._q_translate('') == '_q_index'
        archive_dir()
        archive_dir._q_traverse(['browse_test', ''])
        archive_dir._q_traverse(['browse_test', 'trek.jpg'])
        archive_dir._q_traverse(['browse_test', 'a', 'trek.jpg'])
        contents = archive_dir._q_traverse(['browse_test', 'b', 'c', 'd.txt'])
        assert contents == 'Congratulations!\n'
        publisher._set_request(HTTPRequest(None, dict(SCRIPT_NAME='',
                                                      PATH_INFO='/',
                                                      QUERY_STRING = '40')))
        archive_dir._q_traverse(['browse_test', 'a', 'trek.jpg'])

    def test_image_archive_directory_with_zip(self):
        archive_dir = ImageArchiveDirectory(get_test_archive('zip'))
        publisher = Publisher(archive_dir, logger=PrintLogger())
        publisher._set_request(HTTPRequest(None, dict(SCRIPT_NAME='',
                                                      PATH_INFO='/',
                                                      QUERY_STRING = '40')))
        archive_dir._q_traverse(['browse_test', 'a', 'trek.jpg'])

    def test_image_archive_directory_with_tgz(self):

        archive_dir = ImageArchiveDirectory(get_test_archive('tgz'))
        publisher = Publisher(archive_dir, logger=PrintLogger())
        publisher._set_request(HTTPRequest(None, dict(SCRIPT_NAME='',
                                                      PATH_INFO='/')))

        assert archive_dir._q_translate('') == '_q_index'
        archive_dir._q_traverse(['browse_test', ''])
        archive_dir._q_traverse(['browse_test', 'trek.jpg'])
        archive_dir._q_traverse(['browse_test', 'a', 'trek.jpg'])
        fs = archive_dir._q_traverse(['browse_test', 'b', 'c', 'd.txt'])
        contents = [chunk for chunk in fs]
        assert contents == ['Congratulations!\n']
        publisher._set_request(HTTPRequest(None, dict(SCRIPT_NAME='',
                                                      PATH_INFO='/',
                                                      QUERY_STRING = '40')))
        archive_dir._q_traverse(['browse_test', 'a', 'trek.jpg'])

    def test_index_archive_directory_with_tar(self):

        archive_dir = IndexArchiveDirectory(get_test_archive('tar'))
        publisher = Publisher(archive_dir, logger=PrintLogger())
        publisher._set_request(HTTPRequest(None, dict(SCRIPT_NAME='',
                                                      PATH_INFO='/')))

        assert archive_dir._q_translate('') == '_q_index'
        archive_dir._q_traverse(['browse_test', ''])
        archive_dir._q_traverse(['browse_test', 'trek.jpg'])
        archive_dir._q_traverse(['browse_test', 'a', 'trek.jpg'])
        fs = archive_dir._q_traverse(['browse_test', 'b', 'c', 'd.txt'])
        contents = [chunk for chunk in fs]
        assert contents == ['Congratulations!\n']

if __name__ == "__main__":
    ArchiveDirectoryTest()



"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/cents.py $
$Id: cents.py 25817 2004-12-21 16:27:07Z dbinger $

Provides Cents.
"""
from dulcinea.typeutils import typecheck


class Cents(object):

    """
    Instance attributes:
      cents: int | long
    """
    def __init__(self, value):
        typecheck(value, (int, long))
        self.cents = value

    def get_cents(self):
        return self.cents

    def __str__(self):
        return "$%.02f" % self.get_dollars()

    def get_dollars(self):
        return self.cents / 100.0

    def __add__(self, other):
        typecheck(other, Cents)
        return Cents(self.cents + other.cents)

    def __sub__(self, other):
        typecheck(other, Cents)
        return Cents(self.cents - other.cents)

    def __cmp__(self, other):
        typecheck(other, Cents)
        return cmp(self.cents, other.cents)




#!/www/python/bin/python

"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/test/utest_child_process.py $
$Id: utest_child_process.py 25054 2004-09-09 18:04:30Z dbinger $

Test for dulcinea.child_process.
"""

from sancho.utest import UTest
from dulcinea import child_process


class ChildProcessTest (UTest):

    def check_cat (self):
        p = child_process.execute(('cat',))
        p.write("hello world\n")
        p.stdin.close()
        line = p.readline()
        assert line == 'hello world\n', repr(line)
        assert p.close() == None

    def check_ls (self):
        p = child_process.execute(("ls", "/"))
        assert "bin" in p.read()
        assert p.close() == None

    def check_error (self):
        p = child_process.execute(("false",))
        assert p.close() == 256


if __name__ == "__main__":
    ChildProcessTest()



"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/typeutils.py $
$Id: typeutils.py 22349 2003-08-28 14:13:28Z dbinger $

Provides type-checking utilities: typecheck() and typecheck_seq() so far.
"""

def _typename (typespec):
    if isinstance(typespec, type):
        return typespec.__name__
    elif isinstance(typespec, tuple):
        return ' or '.join([t.__name__ for t in typespec])
    else:
        return 'type %s' % typespec

def typecheck (value, typespec, allow_none=0, isinstance=isinstance):
    """(value : any, typespec : type, allow_none : bool = 0)

    Raises TypeError if 'value' does not match 'typespec'.  'typespec' can
    either be a type or a tuple of types.  If allow_none is true then 'value'
    of None is accepted as well.
    """
    if not ((allow_none and value is None) or isinstance(value, typespec)):
        raise TypeError, \
            "required %s, (got %r of type %s)" % (_typename(typespec),
                                                  value,
                                                  type(value).__name__)

def typecheck_seq (value, typespec, allow_none=0):
    """(value : any, typespec : type, allow_none : bool = 0)

    Raises TypeError if 'value' is a sequence and not every element of the
    sequence matches 'typespec'.  If allow_none is true then 'value' of None
    is accepted as well (None for elements is still disallowed).
    """
    if not allow_none and value is None:
        raise TypeError, "required list, got None"
    elif value is not None:
        for item in value:
            if not isinstance(item, typespec):
                raise TypeError, \
                    "required sequence of %s, (got item %r of type %s)" % \
                            (_typename(typespec), item, type(item).__name__)



"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/code_util.py $
$Id: code_util.py 26358 2005-03-16 15:08:21Z dbinger $
"""
import sys
from cStringIO import StringIO
from types import ModuleType
from compiler.ast import Const, Keyword, Name
from compiler import pycodegen, misc, syntax
from sets import Set
from compiler.ast import Function, Class, Name, AssName, \
     Import, From, ListComp, Lambda, CallFunc
import os.path

# preamble lines are placed in the top of executed source files
# while they are executed to check for coverage.
preamble = ["__name__ = '__main__'"]

try:
    import quixote
    preamble.append('import quixote')
    quixote.enable_ptl()
    preamble.append('quixote.enable_ptl()')
    from quixote.ptl.ptl_compile import parse
except:
    from compiler import parse #n

def get_executable_line_numbers(ast):
    """(ast:AST) -> Set([int])
    Return the set of executable line numbers in the program parsed in `ast`.
    """
    line_numbers = Set()
    def visit(ast, parent):
        if (not isinstance(ast, (Const, Keyword, Name)) or
            (isinstance(ast, Name) and isinstance(parent, CallFunc))):
            line_numbers.add(ast.lineno)
        for node in ast.getChildNodes():
            visit(node, ast)
    visit(ast, None)
    line_numbers.discard(None)
    return line_numbers

def get_trace_data(code):
    '''(code:CodeType) ->
        { filename:str : { why:str : { lineno:int : freq: int} } }
    Executes `code` and returns a dictionary containing the line number
    of every trace callback.
    '''
    results = {}
    saved_trace = sys._getframe().f_trace
    def trace(frame, why, arg):
        filename_dict = results.setdefault(frame.f_code.co_filename, {}) #n
        why_dict = filename_dict.setdefault(why, {}) #n
        lineno = frame.f_lineno #n
        why_dict[lineno] = why_dict.get(lineno, 0) + 1  #n
        return trace #n
    sys.settrace(trace)
    try:
        exec code in {}
    finally:
        sys.settrace(saved_trace)
    return results

def get_line_trace(code, filename):
    trace_data = get_trace_data(code)
    for name in trace_data.keys():
        if (name == filename or
            (os.path.exists(name) and os.path.exists(filename) and
             os.path.samefile(name, filename))):
            return trace_data[name]['line']
    return {}

class ModuleGen(pycodegen.Module):
    """This subclass is here to make sure that quixote's parse
    is used, if it is available.
    """
    def _get_tree(self):
        tree = parse(self.source, self.mode)
        misc.set_filename(self.filename, tree)
        syntax.check(tree)
        self.tree = tree
        return tree

def get_source_file(module):
    """(module:str|ModuleType) -> str
    Identify source file for module.
    """
    if type(module) is str:
        try:
            module_file = __import__(module,{},{},['__file__']).__file__
        except (ImportError, ValueError):
            module_file = module
    else:
        assert type(module) is ModuleType
        module_file = module.__file__
    if module_file[-1] in 'oc':
        module_file = module_file[:-1]
    return os.path.realpath(module_file)

def file_source(module):
    if module is None:
        source_file = '<stdin>'
        raw_source = sys.stdin.read()
    else:
        try:
            source_file = get_source_file(module)
            raw_source = open(source_file, 'rU').read()
        except Exception, e:
            source_file = '<str>'
            raw_source = module
    return source_file, raw_source

def report_missed_lines(main_module, coverage_module=None, verbose=False,
                        out=sys.stdout):
    """(main_module : str|ModuleType|None,
        coverage_module : str|ModuleType|None,
        verbose : bool,
        out : file)
    `main_module` is either the name of a module, a path to a source file, or
    a module.
    If main_module is None, use the source read from stdin.
    This runs the module with __name__ == '__main__' and quixote enabled,
    if it is available.
    Prints a report of lines missed in `coverage_module`, if any are missed.
    """
    main_file, main_source = file_source(main_module)
    source = '\n'.join(preamble) + '\n' + main_source
    module_gen = ModuleGen(source, main_file)
    module_gen.compile()
    if coverage_module is None:
        coverage_file = main_file
        coverage_source = source
        coverage_tree = module_gen.tree
        offset = len(preamble)
        coverage_import_trace = {}
    else:
        coverage_file, coverage_source = file_source(coverage_module)
        coverage_module_gen = ModuleGen(coverage_source, coverage_file)
        coverage_module_gen.compile()
        coverage_tree = coverage_module_gen.tree
        offset = 0
        coverage_import_trace = get_line_trace(coverage_module_gen.code,
                                               coverage_file)

    frequency = {}
    for line_number in get_executable_line_numbers(coverage_tree):
        frequency[line_number] = 0
    frequency.update(coverage_import_trace)

    for line_number, count in get_line_trace(
        module_gen.code, coverage_file).items():
        frequency[line_number] = frequency.get(line_number, 0) + count

    coverage_source_lines = coverage_source.splitlines()
    for index, line in enumerate(coverage_source_lines):
        line_number = index + 1
        if frequency.get(line_number, None) == 0 and "#n" in line:
            del frequency[line_number]

    label_written = False
    for index, line in enumerate(coverage_source_lines):
        line_number = index + 1
        if frequency.get(line_number, None) == 0:
            if label_written == False:
                out.write("missed in %s:\n" % coverage_file)
                label_written = True
            elif frequency.get(line_number - 1, None) != 0:
                out.write("    ...\n")
            out.write("%-4s!%s\n" % (line_number - offset, line))

APPROVED_STARS = ['types'] # names of modules where * imports are ok.

def report_unknown_names(module, verbose=False, out=sys.stdout):
    filename, source = file_source(module)
    unresolved = []
    namespace = Set()

    def arg_names (node):
        result = []
        def flatten(item):
            for x in item:
                if type(x) is tuple:
                    flatten(x)
                else:
                    result.append(x)
        flatten(node.argnames)
        return result

    def traverse(node, namestack):
        childNodes = node.getChildNodes()
        if isinstance(node, Lambda):
            for child in node.defaults:
                traverse(child, namestack)
            prevnode, prevprefix, prevspace = namestack[-1]
            namestack = namestack + [[node, prevprefix + ['<lambda>'],
                                      Set(arg_names(node))]]
            childNodes = [node.code]
        if isinstance(node, Function):
            for child in node.defaults:
                traverse(child, namestack)
            childNodes = [node.code]
            prevnode, prevprefix, prevspace = namestack[-1]
            namestack = namestack + [[node, prevprefix + [node.name],
                                      Set([node.name] + arg_names(node))]]
            if isinstance(prevnode, Class):
                namestack[-1][2].add(prevnode.name)
        if isinstance(node, Class):
            prefix = namestack[-1][1]
            namestack = namestack + [[node, prefix + [node.name], Set([])]]
        if isinstance(node, Name):
            name = node.name
            if (name not in namestack[-1][2] and # local
                name not in namestack[0][2]): # global
                # search nested
                for snode, sprefix, space in namestack[1:-1]:
                    if isinstance(snode, Function) and name in space:
                        break
                else:
                    prefix = namestack[-1][1]
                    unresolved.append((node.lineno, prefix, name))
        if isinstance(node, AssName):
            namestack[-1][2].add(node.name)
            if len(namestack) == 1:
                namespace.add(node.name)
        if isinstance(node, Import):
            for name, as_name in node.names:
                namestack[-1][2].add(as_name or name)
                name_split = name.split('.')
                for index in range(len(name_split)):
                    namespace.add('.'.join(name_split[:index+1]))
        if isinstance(node, From):
            for name, as_name in node.names:
                if name == '*':
                    if verbose:
                        prefix = namestack[-1][1]
                        unresolved.append((node.lineno, prefix, name))
                    if node.modname in APPROVED_STARS:
                        module = __import__(node.modname)
                        names = namestack[-1][2]
                        for name in dir(module):
                            if not name.startswith('_'):
                                names.add(name)
                else:
                    namestack[-1][2].add(as_name or name)
        if isinstance(node, ListComp):
            childNodes = tuple(node.quals +  [node.expr])
        for child in childNodes:
            traverse(child, namestack)
        if isinstance(node, (Function, Class)):
            prevnode, prefix, preset = namestack[-2]
            preset.add(node.name)
            namespace.add('.'.join(prefix + [node.name]))
            if isinstance(node, Class):
                prevnode, prefix, preset = namestack[-1]
                for name in preset:
                    namespace.add('.'.join(prefix + [name]))

    try:
        root_node = parse(source)
    except:
        out.write(filename + '\n')
        raise 
    import __main__
    if type(__builtins__) is dict:
        builtins = __builtins__.keys()
    else:
        builtins = dir(__builtins__) #n
    traverse(root_node,
             [[root_node, [], Set(dir(__main__) + builtins)]])
    for lineno, prefix, name in unresolved:
        if name in namespace:
            if not verbose:
                continue # skip these
            message = "used before definition"
        else:
            message = "not found"
        out.write("%s:%s: %s %s.\n" % (
            filename, lineno, '.'.join(prefix + [name]), message))


def report_unused_imports(module, verbose=False, out=sys.stdout):
    filename, source = file_source(module)
    unused_imports={}
    def walk(ast):
        if isinstance(ast, Import):
            for name, as_name in ast.names:
                if '.' in name:
                    # import a.b
                    # Usage of these names is not currently detected here,
                    # so we won't complain about it.
                    pass
                else:
                    unused_imports[as_name or name] = name
        if isinstance(ast, From):
            for name, as_name in ast.names:
                if '*' != name or verbose:
                    unused_imports[as_name or name] = name
        if isinstance(ast, Name):
            if ast.name in unused_imports:
                del unused_imports[ast.name]
        for child in ast.getChildNodes():
            walk(child)
    walk(parse(source))
    for as_name, name in unused_imports.items():
        if as_name == name:
            out.write("%s: %s\n" % (filename, name))
        if as_name != name:
            if verbose or not as_name.startswith('_'):
                # If you want to import something for a side-effect,
                # but you don't want this warning, make the import
                # an 'as' import and make the name start with underscore.
                out.write("%s: %s as %s\n" % (filename, name, as_name))

def main(report, out=sys.stdout):
    if not sys.argv[1:]:
        out.write("Usage: %s [<file>]+\n" % sys.argv[0])
    for arg in sys.argv[1:]:
        report(arg, out=out)

if __name__ == '__main__':
    def eq(a, b):
        assert a == b, '%r != %r' % (a, b)
    # usage
    sys.argv = ['test']
    s=StringIO()
    main(report_unused_imports, out=s)
    eq(s.getvalue(), 'Usage: test [<file>]+\n')

    # Normal command line.
    sys.argv.append('cgi')
    s=StringIO()
    main(report_unused_imports, out=s)
    eq(s.getvalue(), '')

    # module argument
    import cgi
    s=StringIO()
    report_unused_imports(cgi, out=s)
    eq(s.getvalue(), '')
    # None argument
    s=StringIO()
    input=StringIO('import sys')
    try:
        sys.stdin = input
        report_unused_imports(None, out=s)
    finally:
        sys.stdin = sys.__stdin__
    eq(s.getvalue(), '<stdin>: sys\n')

    # unused import
    s=StringIO()
    report_unused_imports(
        'import sys\n'
        'from types import IntType as num\n'
        'import compiler.ast\n',
        out=s)
    eq(s.getvalue(), '<str>: sys\n<str>: IntType as num\n')

    # unknown names
    s=StringIO()
    report_unknown_names(
        'f(a)\n'
        'def f(x, y=3):\n'
        '    z\n',
        out=s)
    eq(s.getvalue(), '<str>(1): a not found.\n<str>(3): f.z not found.\n')

    # unknown names verbose
    s=StringIO()
    report_unknown_names(
        'f(a)\n'
        'def f(x, y=3):\n'
        '    z\n',
        verbose=True,
        out=s)
    eq(s.getvalue(),
       '<str>(1): f used before definition.\n'
       '<str>(1): a not found.\n'
       '<str>(3): f.z not found.\n')

    # unknown names globals
    s=StringIO()
    report_unknown_names(
        'A=3\n'
        'def f(x, y=3):\n'
        '    return x+A+B+C\n'
        'B=2\n',
        out=s)
    eq(s.getvalue(), '<str>(3): f.C not found.\n')

    # nested
    s=StringIO()
    report_unknown_names(
        'class A:\n'
        '    B=C\n'
        '    def C(self, x=D):\n'
        '        def F(a=x):\n'
        '            return C\n'
        '        return E + B, A\n',
        out=s)
    eq(s.getvalue(),
       '<str>(2): A.C not found.\n'
       '<str>(3): A.D not found.\n'
       '<str>(6): A.C.E not found.\n'
       '<str>(6): A.C.B not found.\n')

    # list comprehensions
    s=StringIO()
    report_unknown_names('[ (x, A) for x in range(3)]', out=s)
    eq(s.getvalue(), '<str>(1): A not found.\n')

    # * imports
    s=StringIO()
    report_unknown_names('from types import *', out=s)
    eq(s.getvalue(), '')
    s=StringIO()
    report_unknown_names('from types import *', verbose=True, out=s)
    eq(s.getvalue(), '<str>(1): * not found.\n')

    # import package
    s=StringIO()
    report_unknown_names('import compiler.ast', out=s)
    eq(s.getvalue(), '')

    # lambda
    s=StringIO()
    report_unknown_names('lambda x, y=a: x+y+z', out=s)
    eq(s.getvalue(),
       '<str>(1): a not found.\n'
       '<str>(1): <lambda>.z not found.\n')

    # missed
    s=StringIO()
    report_missed_lines(
        'def f():\n'
        '    x\n'
        'def g():\n'
        '    x = 1\n'
        '    y\n',
        out=s)
    eq(s.getvalue(),
       'missed in <str>:\n'
       '2   !    x\n'
       '    ...\n'
       '4   !    x = 1\n'
       '5   !    y\n')

    # missed
    s=StringIO()
    import types
    report_missed_lines(
        '\nimport types\n',
        coverage_module=types,
        out=s)

    # unknown names, syntax error
    s=StringIO()
    try:
        report_unknown_names(
            'def f()\n'
            '    x\n',
            out=s)
    except SyntaxError:
        pass

    # test get_executable_line_numbers()
    source = (
        'def f():\n'
        '\n'
        '    x = 3\n'
        '    y = 4\n'
        'f()\n')
    ast = parse(source)
    lines = get_executable_line_numbers(ast)
    assert lines == Set([1, 3, 4, 5])
    module_gen = ModuleGen(source, 'test')
    module_gen.compile()
    trace = get_line_trace(module_gen.code, 'test')
    assert lines == Set(trace.keys())




"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/database.py $
$Id: database.py 26150 2005-02-16 15:00:46Z dbinger $

Provides the ObjectDatabase class.
"""

import sys, os
from durus.file_storage import TempFileStorage
from durus.connection import Connection

class ObjectDatabase:

    """Class used to initialize, access, and close a Durus DB containing a set
    of root objects.
    
    NOTE: Care must be taken to ensure that only one instance exists for each
    Durus database.

    Instance attributes:
      _dbspec : string
        the connection specification for this database
      _current_root_accessors : [str]
      storage : durus.connection.Storage
      connection : durus.connection.Connection
      root : any
    """

    def __init__ (self):
        """ObjectDatabase()

        Creates a new ObjectDatabase object.
        """
        self._dbspec = None
        self.storage = None
        self.connection = None
        self.root = None
        self._current_root_accessors = []

    def open(self, dbspec):
        assert self.root is None, 'database already open'
        self._dbspec = dbspec
        if dbspec.find(":") != -1:
            try:
                (storage_type, args) = dbspec.split(":")
            except ValueError:
                raise ValueError, ("dbspec may have at most one "
                                   "colon: '%s' invalid" % dbspec)
            args = tuple(args.split(","))
        else:
            storage_type = dbspec
            args = ()

        if storage_type == "file":
            (filename,) = args
            self.connection = self._open_file(filename)

        elif storage_type == "client":
            try:
                host, port = args
                port = int(port)
            except ValueError:
                raise ValueError("invalid dbspec: must be "
                                 "'client:<host>,<port>'" % dbspec)
            self.connection = self._open_client((host, port))

        elif storage_type == "empty":
            # leave database and connection as None
            self.storage = TempFileStorage()
            self.connection = Connection(self.storage)
            self.root = {}

        else:
            raise ValueError, "invalid storage type '%s'" % storage_type

        if self.connection is not None:
            self.root = self.connection.get_root()

        self.update_root_accessors()

    def close(self):
        "Close the database connection"
        self._dbspec = None
        self.connection = None
        self.storage = None
        self.root = None

    def get_storage(self):
        return self.storage

    def get_connection(self):
        return self.connection

    def get_root(self):
        return self.root

    def pack(self):
        self.database.pack()

    def _open_file(self, filename):
        """Open 'filename' as a FileStorage, and then open a connection
        around that.  Return a (database, connection) tuple containing the
        DB instance and the Connection instance that results from calling
        its 'open()' method."""

        from durus.file_storage import FileStorage
        created = (not os.path.isfile(filename))
        self.storage = FileStorage(filename)
        if created:
            os.chmod(filename, 0664)            # ensure group writeable
        return Connection(self.storage)

    def _open_client(self, location):
        """Open 'location' (a (hostname, port_number) tuple) as a 
        ClientStorage, and then open a connection around that.  Return
        a (database, connection) tuple."""

        host, port = location
        if host == "":
            # If the specified hostname is the empty string, then
            # 'localhost' is used.
            location = ('localhost', port)

        from durus.client_storage import ClientStorage
        self.storage = ClientStorage(port=port, host=host)
        return Connection(self.storage)

    def get_root_object(self, name):
        assert self.root is not None, "database not initialized"
        if not self.root.has_key(name):
            raise ValueError, "invalid database root '%s'" % name
        return self.root[name]

    def _add_root_accessor(self, name):
        assert not hasattr(self, name)
        setattr(self, name, self.root[name])
        self._current_root_accessors.append(name)

    def init_root(self, name, modulename, klassname):
        __import__(modulename)
        module = sys.modules[modulename]
        klass = getattr(module, klassname)
        assert name not in self.root.keys(), name
        self.root[name] = klass()
        self._add_root_accessor(name)

    def update_root_accessors(self):
        for name in self._current_root_accessors:
            if hasattr(self, name):
                delattr(self, name)
        for name in self.root.keys():
            self._add_root_accessor(name)

    def get_root_names(self):
        return self.root.keys()

    def __iter__(self):
        """
        Yield the objects.
        This works best with a FileStorage.
        """
        for oid, record in self.storage.gen_oid_record():
            yield self.connection[oid]




#!/www/python/bin/python
"""
$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/test/utest_specified_list.py $
$Id: utest_specified_list.py 25054 2004-09-09 18:04:30Z dbinger $
"""

from sancho.utest import UTest
from dulcinea.specified_list import StringList


class Test (UTest):

    def check_stringlist (self):
        s=StringList()
        try:
            s=StringList([1])
            assert 0
        except TypeError: pass
        s=StringList(['a'])
        s.append('b')
        assert list(s) == ['a', 'b']
        s.insert(1, 'c')
        assert list(s) == ['a', 'c', 'b']
        s.insert(0, 'd')
        assert list(s) == ['d', 'a', 'c', 'b']
        try:
            s.insert(0, 3)
            assert 0
        except TypeError: pass
        s.insert(7, 'e')
        assert list(s) == ['d', 'a', 'c', 'b', 'e']
        try:
            s[0] = 7
            assert 0
        except TypeError: pass
        s[0] = 'f'
        assert list(s) == ['f', 'a', 'c', 'b', 'e']
        try:
            s[3:4] = ['a', 4]
            assert 0
        except TypeError: pass
        s[3:4] = ['a', 'b']
        assert list(s) == ['f', 'a', 'c', 'a', 'b', 'e']
        try:
            s.extend(['a', 4])
            assert 0
        except TypeError: pass
        s.extend(['a', 'b'])
        assert list(s) == ['f', 'a', 'c', 'a', 'b', 'e', 'a', 'b']

if __name__ == "__main__":
    Test()



"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/survey.py $
$Id: survey.py 25896 2005-01-19 18:57:50Z rmasse $
"""

from durus.persistent_list import PersistentList
from durus.persistent_dict import PersistentDict
from dulcinea.address import ContactAddress
from dulcinea.keyed import Keyed, KeyedMap
from dulcinea.base import DulcineaBase, DulcineaPersistent
from dulcinea.spec import any, sequence, mapping, require
from dulcinea.timestamped import Timestamped
from dulcinea.typeutils import typecheck, typecheck_seq
from dulcinea.user import DulcineaUser


class Question(Keyed, DulcineaPersistent):
    """
    Instance attributes:
      title : string
        text of the question
      sub_title : string
        sub title
    """

    title_is = any(str, None)
    sub_title_is = any(str, None)

    def __init__(self, title, sub_title=None):
        assert self.__class__ is not Question, "abstract class"
        Keyed.__init__(self)
        self.title = title
        self.sub_title = sub_title

    def __str__(self):
        if self.sub_title:
            return self.title + "\n" + self.sub_title
        return self.title

    def get_title(self):
        return self.title

    def get_sub_title(self):
        return self.sub_title

    def get_new_answer(self):
        raise NotImplementedError, "abstract method"


class TextQuestion(Question):

    """
    Instance attributes:
      dimensions : (rows:int, cols:int) | None
    """
    dimensions_is = (int, int)

    def __init__(self, title, sub_title=None, rows=5, cols=80):
        Question.__init__(self, title, sub_title)
        self.dimensions = (rows, cols)

    def get_new_answer(self):
        return TextAnswer(self)

    def get_dimensions(self):
        # Fix with update_db.
        return getattr(self, 'dimensions', (5, 80))

class ContactAddressQuestion(Question):

    """
    Instance attributes: none
    """
    def get_new_answer(self):
        return ContactAddressAnswer(self)


class SelectQuestion(Question):
    """
    Instance attributes:
      column_headings : [ string ]
        text to place above each column
      row_headings : [ string ]
        text that preceeds each row
      range_labels : [ string ]
        labels to be placed in a row above column headings at the right and left
      last_row_has_text : boolean
        flag to tell the widget to display a TextWidget after the last row label

    A SelectQuestion is an abstract class for representing selections
    from either checkboxes or radio buttons
    """

    column_headings_is = [ any(str, None) ]
    row_headings_is = [ any (str, None) ]
    range_labels_is = [ str ]
    last_row_has_text_is = bool

    def __init__(self, title, sub_title=None,
                 column_headings=None, row_headings=None,
                 range_labels=None, last_row_has_text=False):
        Question.__init__(self, title, sub_title=sub_title)
        self.column_headings = column_headings or []
        self.row_headings = row_headings or []
        self.range_labels = range_labels or []
        assert len(self.range_labels) in (0,2), "must have two range labels"
        self.last_row_has_text = last_row_has_text

    def __str__(self):
        question = self.title
        if len(self.column_headings) > 1: # Radioset
            if len(self.column_headings) > 4:
                headings = '(%s ... %s)' % (self.column_headings[0],
                                            self.column_headings[-1])
            else:
                headings = '(%s)' % (', '.join(self.column_headings))
            question += "\n" + headings
            if self.range_labels:
                question += "\n  ...where %s is %s and %s is %s" % (
                    self.column_headings[0], self.range_labels[0],
                    self.column_headings[-1], self.range_labels[-1])
        if self.sub_title:
            question += "\n" + self.sub_title
        return question

    def get_row_headings(self):
        return self.row_headings

    def get_column_headings(self):
        return self.column_headings

    def get_range_labels(self):
        return self.range_labels

    def get_new_answer(self):
        return SelectAnswer(self)


class Answer(DulcineaBase):
    """
    Instance attributes:
      question : Question
        the question associated with this answer
    """

    question_is = Question

    def __init__(self, question):
        assert self.__class__ is not Answer, "abstract class"
        self.question = question

    def get_question(self):
        return self.question



class ContactAddressAnswer(Answer):
    """
    Instance attributes:
      answer : ContactAddress
    """

    answer_is = ContactAddress

    def __init__(self, question):
        Answer.__init__(self, question)
        self.answer = ContactAddress()

    def __str__(self):
        return self.answer.format()

    def get_answer(self):
        return self.answer

    def set_answer(self, answer):
        self.answer = answer

    def has_answer(self):
        return self.answer != ContactAddress()



class TextAnswer(Answer):
    """
    Instance attributes:
      answer : string
        text answer to a question
    """

    answer_is = str

    def __init__(self, question):
        Answer.__init__(self, question)
        self.answer = None

    def __str__(self):
        return self.answer

    def get_answer(self):
        return self.answer

    def set_answer(self, answer):
        self.answer = answer

    def has_answer(self):
        return self.answer is not None

class SelectAnswer(Answer):
    """
    Instance attributes:
      selected : [ string | boolean ]
        list of selected row answers
      other_text : string
        contents of the filled in 'last_row_has_text' textfield
    """
    selected_is = [any(str, bool)]

    def __init__(self, question):
        Answer.__init__(self, question)
        self.selected = None
        self.other_text = None

    def __str__(self):
        if self.question.row_headings:
            def format_heading_answer(heading, answer):
                if answer:
                    return "%s: %s" % (heading, answer)
                return ''
            return "\n".join([format_heading_answer(*heading_answer) for
                              heading_answer in
                              zip(self.question.row_headings, self.selected)])
        return ", ".join(self.selected)

    def set_other_text(self, other_text):
        self.other_text = other_text

    def get_other_text(self):
        return self.other_text

    def set_answer(self, selected):
        self.selected = selected

    def get_answer(self):
        return self.selected

    def has_answer(self):
        return self.selected is not None


class Survey(DulcineaPersistent, Timestamped):
    """A set of Questions and Answers attached to a single object.

    Instance attributes:
      user : DulcineaUser
        user taking the survey
      answers : [ Answer ]
        answers to the questions that comprise the object's survey
    """

    user_is = any(DulcineaUser, None)
    answers_is = [Answer]

    def __init__(self, answers):
        """(answers:[Answer])
        """
        Timestamped.__init__(self)
        typecheck_seq(answers, Answer)
        self.answers = answers
        self.user = None

    def __str__(self):
        return "By %s on %s: %d answers" % (
            self.user, self.get_timestamp(), len(self.answers))

    def __len__(self):
        return len(self.answers)

    def get_questions(self):
        return [answer.get_question() for answer in self.answers]

    def get_answers(self):
        return self.answers

    def get_answer_for_question_key(self, question_key):
        for answer in self.answers:
            if answer.get_question().get_key() == question_key:
                return answer
        return None

    def get_user(self):
        return self.user

    def set_user(self, user):
        """(user : DuclineaUser)
        """
        typecheck(user, DulcineaUser, allow_none=1)
        self.user = user

    def set_answers(self, answers, user):
        """(answers : [Answer], user : DulcineaUser)
        """
        typecheck_seq(answers, Answer)
        self.answers = answers
        self.set_user(user)
        self.set_timestamp()

    def has_answers(self, omit_questions=None):
        """(omit_questions : [int] | None) -> boolean

        Return true if any answers have been answered for this survey
        ignoring omitted questions if specified
        """
        for answer in self.answers:
            if answer.get_question().get_key() in (omit_questions or []):
                continue
            if answer.has_answer():
                return True
        return False


class Surveyable:
    """A mixin for objects that have Surveys

    Instance attributes:
      surveys : PersistentList [ Survey ]
    """

    surveys_is = any(sequence(Survey, PersistentList), None)

    def __init__(self):
        self.surveys = None

    def get_surveys(self):
        """() -> [Survey]
        """
        return self.surveys or []

    def get_survey(self, index):
        return self.surveys and self.surveys[index]

    def get_new_survey(self):
        """() -> Survey
        """
        raise NotImplementedError, "abstract method"

    def add_survey(self, survey):
        if self.surveys is None:
            self.surveys = PersistentList()
        self.surveys.append(survey)


class QuestionDatabase(KeyedMap, DulcineaPersistent):
    """
    Instance attributes: none
    """
    def __init__(self):
        KeyedMap.__init__(self, value_spec=Question)

    def get_answers(self, question_keys=None):
        """(question_keys: [int]) -> [Answer]
        """
        if not question_keys:
            question_keys = self.get_mapping().keys()
            question_keys.sort()
        return [self.get(question_key).get_new_answer()
                for question_key in question_keys]


class SurveysAndQuestions(Surveyable, QuestionDatabase):
    """
    Instance attributes:
      id : str
      description : str
      title : str
    """

    id_is = str
    title_is = str
    description_is = str

    def __init__(self, id, questions):
        """(id : str, questions : [Question])
        """
        Surveyable.__init__(self)
        QuestionDatabase.__init__(self)
        self.id = id
        for question in questions:
            self.add(question)
        self.title = ''
        self.description = ''

    def get_id(self):
        return self.id

    def set_title(self, title):
        require(title, self.title_is)
        self.title = title

    def get_title(self):
        return self.title

    def set_description(self, description):
        require(description, self.description_is)
        self.description = description

    def get_description(self):
        return self.description

    def get_new_survey(self):
        return Survey(self.get_answers())


class SurveysAndQuestionsDatabase(DulcineaPersistent):
    """
    Instance attributes:
      surveys_and_questions :  PersistentDict { str : SurveysAndQuestions }
    """

    surveys_and_questions_is = mapping({str:SurveysAndQuestions},
                                       PersistentDict)

    def __init__(self):
        self.surveys_and_questions = PersistentDict()

    def add(self, surveys_and_questions):
        require(surveys_and_questions, SurveysAndQuestions)
        id = surveys_and_questions.get_id()
        assert id not in self.surveys_and_questions, id
        self.surveys_and_questions[id] = surveys_and_questions

    def get(self, id):
        return self.surveys_and_questions[id]

    def get_mapping(self):
        return self.surveys_and_questions




#!/www/python/bin/python
"""
$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/test/utest_calendar.py $
$Id: utest_calendar.py 25054 2004-09-09 18:04:30Z dbinger $
"""

from datetime import datetime, timedelta
from sancho.utest import UTest
from dulcinea.user import DulcineaUser
from dulcinea.calendar import Calendar, Reservation, Reservable, Resources, \
     Resource, weekdays


class Test (UTest):

    def check_a(self):
        self.calendar = Calendar()
        self.block1 = Reservation()
        try:
            self.calendar.add(self.block1)
            assert 0
        except TypeError: pass
        title = 'test1'
        self.block1.set_title(title)
        assert self.block1.get_title() == title
        self.block1.set_description(title)
        assert self.block1.get_description() == title
        reserved = Resource()
        self.block1.set_reserved(reserved)
        assert self.block1.get_reserved() == reserved
        user = DulcineaUser()
        self.block1.set_reserver(user)
        start1 = datetime(1,1,1)
        self.block1.set_start_time(start1)
        assert self.block1.get_start_time() == start1
        end1 = start1 + timedelta(days=1)
        self.block1.set_end_time(end1)
        assert self.block1.get_end_time() == end1

        self.calendar.add(self.block1)
        assert self.calendar.get(self.block1.get_key()) == self.block1
        assert self.calendar.get('', None) == None
        assert self.calendar.get_mapping() == {
            self.block1.get_key():  self.block1 }

        self.reservable = Reservable()
        self.reservable.set_active(True)
        assert self.reservable.is_active() == True
        title = 'room 1'
        self.reservable.set_title(title)
        assert self.reservable.get_title() == title
        self.block1.set_reserved(self.reservable)

        assert self.reservable.allows_control(user) == False
        assert self.reservable.allows_reserve(user) == False
        class AdminUser(DulcineaUser):
            def is_admin(self):
                return True
        admin_user = AdminUser()
        assert self.block1.can_modify(None) == False
        assert self.block1.can_modify(admin_user) == True

        self.block1.set_reserver(admin_user)
        assert self.block1.can_modify(user) == False
        self.reservable.set_permission('control', user)
        assert self.block1.can_modify(user) == True

        user2 = DulcineaUser()
        self.block1.set_reserver(user2)
        assert self.block1.can_modify(user2) == True

        Resources()

        time_increment = timedelta(hours=2)
        start_time = start1
        end_time = start_time + timedelta(days=3)
        start_time_of_day = timedelta(hours=8)
        end_time_of_day = timedelta(hours=18)
        resource = Resource()
        reservation = Reservation()
        reservation.set_title("")
        reservation.set_description("")
        reservation.set_reserved(resource)
        reservation.set_reserver(user)
        reservation.set_start_time(start_time + start_time_of_day)
        reservation.set_end_time(start_time + start_time_of_day + 
                                 2 * time_increment)
        reservation.set_reserved(resource)
        self.calendar.add(reservation)
        self.calendar.get_table_map(
            resource, time_increment, start_time, end_time, 
            start_time_of_day, end_time_of_day)


    def check_weekdays(self):
        failures = False
        for length in range(9):
            for offset in range(9):
                start = datetime(2000, 1, 1) + timedelta(days=offset)
                end = start + timedelta(days=length)
                weekdays1 = weekdays(start, end)
                currentday = start
                weekdays2 = 0
                for i in range(length):
                    if 5 <= currentday.weekday() <= 6:
                        pass
                    else:
                        weekdays2 += 1
                    currentday += timedelta(days=1)
                if weekdays1 != weekdays2:
                    failures = True
        assert not failures

if __name__ == "__main__":
    Test()



#!/www/python/bin/python

"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/test/utest_typeutils.py $
$Id: utest_typeutils.py 25054 2004-09-09 18:04:30Z dbinger $

Test for dulcinea.typeutils.
"""

from types import *
from sancho.utest import UTest
from dulcinea.typeutils import typecheck, typecheck_seq


class TypeUtilsTest (UTest):

    def check_typecheck_seq (self):
        typecheck_seq([4, 2], IntType)
        try:
            typecheck_seq([4.0, 2], IntType)
            assert 0
        except TypeError: pass
        try:
            typecheck_seq([4.0, 2], FloatType)
            assert 0
        except TypeError: pass
        typecheck_seq([4.0, 2.0], FloatType)
        typecheck_seq(['foo', 'bar', 'baz'], str)
        try:
            typecheck_seq(['foo', 'bar', 666], str)
            assert 0
        except TypeError: pass
        typecheck_seq([], str)
        typecheck_seq([], FloatType)
        typecheck_seq([], IntType)
        try:
            typecheck_seq(4, IntType)
            assert 0
        except TypeError: pass
        try:
            typecheck_seq(None, IntType)
            assert 0
        except TypeError: pass
        typecheck_seq(None, IntType, allow_none=1)

    def check_typecheck (self):
        typecheck(4, IntType)
        try:
            typecheck(4.0, IntType, allow_none=1)
            assert 0
        except TypeError: pass
        typecheck(4.0, (FloatType, IntType))
        try:
            typecheck('foo', (FloatType, IntType))
            assert 0
        except TypeError: pass
        try:
            typecheck(None, IntType)
            assert 0
        except TypeError: pass
        typecheck(None, IntType, allow_none=1)

if __name__ == "__main__":
    TypeUtilsTest()



"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/contact.py $
$Id: contact.py 25629 2004-11-17 17:54:48Z nascheme $
"""

from dulcinea import local
from dulcinea.address import Addressable, ContactAddress
from dulcinea.base import DulcineaPersistent
from dulcinea.history import Historical
from dulcinea.permission import PermissionManager
from dulcinea.user import DulcineaUser, DulcineaUserDatabase
from dulcinea.typeutils import typecheck

class Contact(DulcineaUser, Addressable, Historical):
    """
    A more heavyweight user object that contains an Address object.
    Expects on a PermissionManager to determine if the user has admin power.

    Instance attributes:
      email : string
        the user's Internet email address
      prefix : string
      first_name : string
      last_name : string
      phone : string
      fax : string                  
      company_name : string
    """

    def __init__(self, user_id=None):
        DulcineaUser.__init__(self, user_id=user_id)
        Addressable.__init__(self)
        Historical.__init__(self)
        self.email = None
        self.prefix = None
        self.first_name = None
        self.last_name = None
        self.company_name = None
        self.phone = None
        self.fax = None

    def get_contact_address(self):
        address = self.get_address()
        return ContactAddress(
            contact_name=self.get_contact_name(),
            contact_phone_number = self.get_phone(),
            company_name=self.get_company_name(),
            **address.__dict__)

    def get_contact_name(self, limit=35):
        last_name = self.last_name or ''
        first_name = self.first_name
        contact_name = last_name
        if first_name and len(contact_name) <= limit:
            if limit - len(contact_name) > len(first_name) + 1:
                contact_name = first_name + ' ' + last_name
            elif limit - len(contact_name) > 2:
                contact_name = first_name[0] + ' ' + last_name
        return contact_name[:limit]

    def get_company_name(self):
        return self.company_name

    def set_company_name(self, company_name):
        typecheck(company_name, str, allow_none=1)
        self.company_name = company_name

    def format_realname(self):
        first_name = self.first_name or ''
        last_name = self.last_name or ''
        if first_name and last_name:
            return first_name + ' ' + last_name
        else:
            return first_name or last_name

    def format(self):
        """() -> string

        Return a string representing this user: either their real name
        (as returned by 'format_realname()') or their user ID.
        Guaranteed to return a non-empty string.
        """
        return self.format_realname() or self.id

    def set_prefix(self, prefix):
        typecheck(prefix, str, allow_none=True)
        self.prefix = prefix

    def get_prefix(self):
        return self.prefix

    def set_first_name(self, first_name):
        typecheck(first_name, str, allow_none=True)
        self.first_name = first_name

    def get_first_name(self):
        return self.first_name

    def set_last_name(self, last_name):
        typecheck(last_name, str, allow_none=True)
        self.last_name = last_name

    def get_last_name(self):
        return self.last_name

    def set_email(self, email):
        typecheck(email, str, allow_none=True)
        self.email = email

    def get_email(self):
        return self.email

    def set_phone(self, phone):
        typecheck(phone, str, allow_none=True)
        self.phone = phone

    def get_phone(self):
        return self.phone

    def set_fax(self, fax):
        typecheck(fax, str, allow_none=True)
        self.fax = fax

    def get_fax(self):
        return self.fax

    def get_email(self):
        return self.email

    def is_admin(self):
        return local.get_user_db().get_admin().directly_grants(
            'staff', self)

    def record_registration (self, host):
        self.history.add_event(None, 'register',
                               'new user ID registered from host %s' % host)

    def record_login (self, host):
        self.history.add_event(None, 'login', 'login from host %s' % host)

    def record_logout (self, host):
        self.history.add_event(None, 'logout', 'logout from host %s' % host)

    def record_failed_login (self, host):
        self.history.add_event(None, 'failed_login',
                               'failed login attempt from host %s' % host)

    def record_change (self, user, host, msg=None, password=0):
        code = password and 'change_password' or 'change'
        if msg is None:
            if password:
                msg = 'changed password'
            else:
                msg = 'changed other data'
        self.history.add_event(user, code, msg + ' (host %s)' % host)

    def record_act_as_start (self, user):
        self.history.add_event(user, 'act_as_start',
                               'Actual user %s began acting '
                               'as this user' % user.id)

    def get_registration_time (self):
        """() -> datetime | None
        """
        return self.history.find_last_date('register')

    def get_last_login_time (self):
        """() -> datetime | None"""
        return self.history.find_last_date('login')


class ContactAdmin(DulcineaPersistent, PermissionManager):
    """
    Instance attributes: none
    """
    valid_permissions = {
        "manage-permissions":
            "Allow changing of permissions.",
        "staff":
            ("Is a member of the staff, with all of the privileges and "
             "responsibilities thereunto appertaining."),
        "act-as":
            "Allow to act as another user.",
        "system":
            "Allow to do things normally done by the software system.",
        "create-users":
            "Allow the creation of other users."
        }

    def __init__(self):
        PermissionManager.__init__(self)

    def __str__(self):
        return 'admin'



class ContactDatabase(DulcineaUserDatabase):
    """
    Instance attributes:
      admin: ContactAdmin
      users : PersistentDict { user_id:string : user:Contact }
    """

    user_class = Contact

    def __init__(self, admin=None):
        DulcineaUserDatabase.__init__(self)
        system_user = self.user_class("SYSTEM")
        self.add_user(system_user)
        self.admin = admin or ContactAdmin()
        self.admin.set_permission('system', system_user)

    def get_admin(self):
        return self.admin



#!/www/python/bin/python

"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/test/utest_permission.py $
$Id: utest_permission.py 25054 2004-09-09 18:04:30Z dbinger $

Test for dulcinea.permission.
"""

from sets import Set
from sancho.utest import UTest
from dulcinea.permission import PermissionManager


class WaitPass (PermissionManager):

    valid_permissions = {'pass' : 'just pass',
                         'wait' : 'just wait'}


class PermissionManagerTest (UTest):

    def check_recursion_control(self):
        self.s=s=WaitPass()
        self.r=r=WaitPass()
        self.q=q=WaitPass()
        s.set_permission('pass', r, prerequisite='pass')
        r.set_permission('pass', q, prerequisite='pass')
        q.set_permission('pass', s, prerequisite='pass')
        assert not s.grants('pass', None)
        assert not r.grants('pass', None)
        assert not q.grants('pass', None)
        s.set_permission('pass', s, prerequisite='pass')
        assert not s.grants('pass', None)
        assert not r.grants('pass', None)
        assert not q.grants('pass', None)

    def check_all(self):
        s = WaitPass()
        r = WaitPass()
        q = WaitPass()
        # None is used in this test as the object for which
        # permissions are applied.  Permissions can be granted to
        # any hashable object.
        assert not s.grants('pass', None)
        assert not s.directly_grants('pass', None)
        s.set_permission('pass', None)
        s.set_permission_items('pass', [(None,None)])
        assert s.directly_grants('pass', None)

        # None shall pass.  s grants 'pass' to None.
        assert s.grants('pass', None)
        # Set that r grants 'wait' if s grants 'pass'
        r.set_permission('wait', s, prerequisite='pass')

        # Check get_direct_grantees
        assert r.get_direct_grantees('wait') == []
        assert s.get_direct_grantees('pass') == [None]

        # r does not grant 'pass'
        assert not r.grants('pass', None)
        # r grants 'wait' to None because s grants 'pass' to None.
        assert r.grants('wait', None)
        # Set that q grants 'pass' if r grants 'wait'.
        q.set_permission('pass', r, prerequisite='wait')
        # q grants 'pass' to None because r grants 'wait' to None,
        # in turn because s grants 'pass' to None.
        assert q.grants('pass', None)

        # Check accessor
        assert Set(q.get_valid_permissions().keys()) == Set(['pass', 'wait'])
        assert r.get_permissions(None) == ['wait']
        assert s.get_permissions(None) == ['pass']
        assert s.get_permission_items('pass') == [(None, None)]
        assert s.get_permission_items('wait') == []

        assert not s.grants('nonsense', None)
        WaitPass().set_permission_items('wait', [])
        WaitPass().get_direct_grantees('wait')




if __name__ == "__main__":
    PermissionManagerTest()




#!/usr/bin/env python
"""
$URL: svn+ssh://svn/repos/trunk/dulcinea/bin/unknown.py $
$Id: unknown.py 26302 2005-03-08 14:06:55Z dbinger $
"""

from dulcinea.code_util import report_unknown_names, main

main(report_unknown_names)



#!/www/python/bin/python

"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/test/utest_user.py $
$Id: utest_user.py 25054 2004-09-09 18:04:30Z dbinger $

Test for dulcinea.user.
"""

from sancho.utest import UTest
from dulcinea.user import DulcineaUser as User


class DulcineaUserTest (UTest):

    def check_init (self):
        "Check constructor"
        try:
            User('!@#$%^')
            assert 0
        except ValueError: pass
        User('amk')
        u = User() ; u.set_id('foo')
        try:
            User('foo').set_id('foo')
            assert 0
        except AssertionError: pass
        try:
            u = User() ; u.set_id('!foo')
            assert 0
        except ValueError: pass

        u = User()
        assert u.id == None
        u.set_id('foo')
        assert u.id == "foo"
        try:
            u.set_id('bar')
            assert 0
        except AssertionError: pass

    def check_password (self):
        "Test password-related methods"
        user = User('foo')
        user.set_password("biteme")
        assert user.valid_password("biteme")
        assert not user.valid_password("dontbiteme")
        password1 = user.generate_password()
        assert user.valid_password(password1)
        password2 = user.generate_password()
        assert user.valid_password(password2)
        assert password1 != password2


if __name__ == "__main__":
    DulcineaUserTest()



#!/www/python/bin/python
"""
$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/test/utest_keyed.py $
$Id: utest_keyed.py 25054 2004-09-09 18:04:30Z dbinger $
"""

from sancho.utest import UTest
from dulcinea.keyed import Keyed, KeyedMap


class Test (UTest):

    def check_a(self):
        self.map = KeyedMap()
        self.a = Keyed()
        assert self.a.get_key() == None
        self.map.add(self.a)
        try:
            self.map.add(self.a)
            assert 0
        except AssertionError: pass
        self.b = Keyed()
        self.b.junk = 1
        try:
            self.map.add(self.b)
            assert 0
        except TypeError: pass
        try:
            self.map.add(1)
            assert 0
        except TypeError: pass
        assert self.a.get_key() == 1
        assert self.map.get_mapping().items() == [(1, self.a)]
        assert self.map.get(1) == self.a

if __name__ == "__main__":
    Test()



"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/traversal.py $
$Id: traversal.py 26248 2005-02-25 14:56:37Z dbinger $
"""

from quixote import enable_ptl
enable_ptl()
from sets import Set as set
from quixote import get_session
from dulcinea.site_util import get_root_directory
from dulcinea.ui.crumbs import get_exports
from dulcinea.ui.directory import DynamicExportingDirectory
from dulcinea.ui.publisher import DulcineaPublisher
from quixote.directory import Directory
from quixote.http_request import HTTPRequest
from dulcinea import local


def check_exports():
    db = local.open_database()
    publisher = DulcineaPublisher()
    publisher._set_request(HTTPRequest(None, dict(SCRIPT_NAME='',
                                                  PATH_INFO='/')))
    admin_user = local.get_user_db().get_admin().get_direct_grantees(
        'staff')[0]
    publisher.start_request()
    get_session().user = admin_user
    todo = set([get_root_directory()])
    seen = set()
    while todo:
        directory = todo.pop()
        seen.add(directory)
        if isinstance(directory, DynamicExportingDirectory):
            if not hasattr(directory, 'get_exports'):
                print directory, 'is missing get_exports()'
        else:
            if not hasattr(directory, '_q_exports'):
                print directory, 'is missing _q_exports'
        if (getattr(directory, '_q_exports', None) and
            getattr(directory, 'get_exports', None)):
            print directory, 'has both _q_exports and get_exports()'
        unused_q_index = hasattr(directory, '_q_index')
        for component, export, crumb, title in get_exports(directory):
            if component == "" and export == '_q_index':
                unused_q_index = False
            if not hasattr(directory, export):
                if hasattr(directory, '_q_resolve'):
                    print directory, '_q_resolve must provide', export
                else:
                    print directory, 'is missing', export
            else:
                if isinstance(getattr(directory, export, None),
                              Directory):
                    todo.add(getattr(directory, export, None))
        if unused_q_index:
            print directory, 'has an unused _q_index'
    db.close()
    import quixote.publish
    quixote.publish._publisher = None
